
From: Maneesh Soni <maneesh@in.ibm.com>

The patch fixes locking in proc_check_root().  It brings is_subdir() call
under vfsmount_lock.  Holding vfsmount_lock will ensure mnt_mountpoint
dentry is intact and the dentry does not go away while it is being checked
in is_subdir().  



---

 fs/proc/base.c |    7 +++----
 1 files changed, 3 insertions(+), 4 deletions(-)

diff -puN fs/proc/base.c~proc_check_root-locking-fix fs/proc/base.c
--- 25/fs/proc/base.c~proc_check_root-locking-fix	2004-01-27 23:50:05.000000000 -0800
+++ 25-akpm/fs/proc/base.c	2004-01-27 23:50:05.000000000 -0800
@@ -425,17 +425,15 @@ static int proc_check_root(struct inode 
 	mnt = vfsmnt;
 
 	while (vfsmnt != our_vfsmnt) {
-		if (vfsmnt == vfsmnt->mnt_parent) {
-			spin_unlock(&vfsmount_lock);
+		if (vfsmnt == vfsmnt->mnt_parent)
 			goto out;
-		}
 		de = vfsmnt->mnt_mountpoint;
 		vfsmnt = vfsmnt->mnt_parent;
 	}
-	spin_unlock(&vfsmount_lock);
 
 	if (!is_subdir(de, base))
 		goto out;
+	spin_unlock(&vfsmount_lock);
 
 exit:
 	dput(base);
@@ -444,6 +442,7 @@ exit:
 	mntput(mnt);
 	return res;
 out:
+	spin_unlock(&vfsmount_lock);
 	res = -EACCES;
 	goto exit;
 }

_
